#-------------------------------------------------------------------------------
# Copyright 2009  Swiss Federal Institute of Aquatic Science and Technology
#
# This file is part of UWIM (Urban Water Infrastructure Model).
#
# UWIM is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# UWIM is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWIM.  If not, see <http://www.gnu.org/licenses/>.
#
# contact: max.maurer@eawag.ch
#===============================================================================
#
# Urban Water Infrastructure Model (UWIM), static version
#
# Functions to generate objects of the class "Gebiet" 
# and to plot or to print those objects.
#
# only SI-units are used
# Version 0.3.s
# 15.5.2009
#===============================================================================

#===============================================================================
# generate an object of the class "Gebiet"
#===============================================================================

Gebiet.Gen = function (P) {

#----------------------
# 1. geometry
#----------------------      

Geometrie=list(
      E=P$E,                		#number of inhabitants
      A=P$A,                		#size of the catchment area [m2]
      A.L=P$A/P$Anz.G,     			#size of a housing plot [m2]      
      Anz.G=P$Anz.G,        		#number of buildings 
      B.Q=sqrt(P$A/P$f1),   		#width of the catchment area [m]
      L.Q=sqrt(P$A*P$f1),          	#length of the catchment area [m]
      B.L=sqrt( (P$A/P$Anz.G)/P$f2 ), 	#width of a housing plot [m]
      L.L=sqrt( (P$A/P$Anz.G)*P$f2 ),	#length of a housing plot [m]
      TK=P$TK					#total length of intercepting sewer [m]
)

#calculate all lengths
Out.Laenge=Laenge(P, Geometrie)
       
#----------------------
# 2. bore dimension
#----------------------  

#calculate the necessary bores
Out.NW=Liste(P, Out.Laenge)

##----------------------
## 4. cost calculation
##---------------------- 

Out.Kosten=Kosten(P, Out.NW, Out.Laenge) 


##----------------------
## 5. return
##----------------------

rueck=list(
       P=P,                 #save all parameters 
       D.HS=Out.NW$D.HS,
       D.O=Out.NW$D.O,
       D.U=Out.NW$D.U,
       NW.Verteilung=NW.Verteilung(Out.NW, Out.Laenge$n.Ent.S),
       n.Ent.S=Out.Laenge$n.Ent.S,
       D.Ent.S=2*Geometrie$L.L,
       L.HS.H.eff=Out.Laenge$L.HS.H.eff,
       L.Ent.H.eff=Out.Laenge$L.Ent.H.eff,
       n.Ent.H1=Out.Laenge$n.Ent.H1,
       n.Ent.H2=Out.Laenge$n.Ent.H2,
       Laenge.oeff=Out.Laenge$L.tot.oeff,
       Laenge.privat=Out.Laenge$L.tot.privat,
       Kosten=Out.Kosten$Kosten.tot
      ) 

class(rueck)="Gebiet"
return(rueck)


}

#===============================================================================
# plot of the bore distribution
#===============================================================================

plot.Gebiet = function(Gebiet) {
      
      #all available diameters [m]
      alle=data.frame(Var1=c(.25, .35, .4, .5, .6, .8, 1, 1.2, 1.5, 2, 2.5, 3, 4, 6, 8))
      
      #length per bore (sewer lines) [m]  
      a=data.frame(table(c(Gebiet$D.O$NW, Gebiet$D.U$NW)))           
      a[nrow(a),2]=a[nrow(a),2]-1                           #(cf. equation (14))
      a[,2]=a[,2]*Gebiet$D.O[1,2]*Gebiet$n.Ent.S            #length per bore
      
      #length per bore (main sewer) [m]           
      b=data.frame(table(Gebiet$D.HS$NW)*Gebiet$D.HS[1,2])

      #merge
      M=merge(alle,a,by.x="Var1",by.y="Var1", all=T) 
      M=merge(M,b,by.x="Var1",by.y="Var1", all=T)   
    
      #length per bore (all pipesections) [m] 
      Laenge=rowSums(M[,2:3], na.rm=T)
      names(Laenge)=M[,1]

      #cum. rel. distribution
      Laenge.cum=cumsum(Laenge/sum(Laenge))
      
      # -- plot --
      par(las=1, xpd=F, mar=c(5,4,4.5,2)+0.1)        
      plot(Laenge.cum*100, type="s", ylim=c(0,100), xlim=c(0.8, length(Laenge)+0.3), ylab="proportion", xaxt='n',
              xlab="diameter [m]", main="Bore Distribution", lwd=2)
      
      #linies
      abline(h=seq(0,1,0.1)*100, col=gray(0.5), lty=3)
      abline(h=c(0,0.5,1)*100, col=gray(0.5), lty=1)
      
      #rel. proportion       
      points(Laenge*100/sum(Laenge),col=2, type="h")
      points(Laenge*100/sum(Laenge),col=2, pch=16)
      
      #labelling of the axes  
      par(las=3)
      axis(1, at=1:length(Laenge.cum),   labels=names(Laenge))
      
      #labelling the lengths
      text(Laenge*100/sum(Laenge)+2.5, labels=paste(round(Laenge),"m",sep=""),
               pos=4, cex=0.8, srt=60)
      
      #labelling the total length
      par(las=1)
      p.laeng=formatC(round(Gebiet$Laenge.oeff,-1), big.interval=3, big.mark="'", format="d")
      mtext(paste("total length:", p.laeng, "m"), at=2, side=1, padj=5.7)


      #legend 
      par(xpd=NA)
      tmp.u = par('usr') 
      legend(tmp.u[1], tmp.u[4], xjust=0, yjust=0, 
              legend=c("cumulative distribution", "relative distribution"), 
              col=c(1,2), lwd=c(2,1), horiz=T, bty='n', cex=0.8)
      
      par(xpd=FALSE)
              
}

#===============================================================================
# print funktion
#===============================================================================

print.Gebiet= function (Gebiet, alles=FALSE) {

       #output as list
       if(alles==TRUE) NextMethod()
       
       #simplified output
       if(alles==FALSE){
                    
              #format big numbers            
              K=formatC(round(Gebiet$Kosten,-0), big.interval=3, big.mark="'", format="d")
                          
              
              #output
              cat("\n general data: \n",
                   "area [ha]:", round(Gebiet$P$A/10000,2), "\t buildings:", round(Gebiet$P$Anz.G,1), "\t inhabitants:", round(Gebiet$P$E), "\n",
                   "\n sewer system: \n",
                   "total length public [m]:", round(Gebiet$Laenge.oeff), "\t length interconnecting drains [m]:", Gebiet$P$TK, "\t total length public private [m]:", 
		     			round(Gebiet$Laenge.privat), "\n\n",
                   Gebiet$n.Ent.H1, "pipes above main trunk sewer, ", Gebiet$n.Ent.H2, "pipes below m.t.s. \n",
                   round(Gebiet$n.Ent.S,2), "secundary sewer lines \n", 
                   "length of one pipe in the secundary sewer line [m]:\t", round(Gebiet$L.Ent.H.eff,2), "\n",
                   "length of one pipe in the main trunk sewer [m]:\t", round(Gebiet$L.HS.H.eff,2), "\n \n",
                   
                   "costs: \n",
                   "replacement costs", K, "SFr. \n",
		     "replacement costs per capita", round(Gebiet$Kosten/Gebiet$P$E), "SFr. \n" 
                   )
    
       }
}








